###
# Copyright (c) 2011, Rebecca Bettencourt
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

###

import os
import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

class SqliteReadingListDB(object):
    def __init__(self, filename):
        self.dbs = ircutils.IrcDict()
        self.filename = filename

    def close(self):
        for db in self.dbs.itervalues():
            db.close()

    def _getDb(self, channel):
        try:
            import sqlite
        except ImportError:
            raise callbacks.Error, 'You need to have PySQLite installed to ' \
                                   'use ReadingList.  Download it at ' \
                                   '<http://pysqlite.org/>'
        filename = plugins.makeChannelFilename(self.filename, channel)
        if filename in self.dbs:
            return self.dbs[filename]
        if os.path.exists(filename):
            self.dbs[filename] = sqlite.connect(filename)
            return self.dbs[filename]
        db = sqlite.connect(filename)
        self.dbs[filename] = db
        cursor = db.cursor()
        cursor.execute("""CREATE TABLE readings (
                          id INTEGER PRIMARY KEY,
                          category INTEGER,
                          url TEXT
                          )""")
        cursor.execute("""CREATE TABLE readers (
                          id INTEGER,
                          name TEXT,
                          normalized TEXT
                          )""")
        db.commit()
        return db

    def add_reading(self, channel, category, name, url):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT id FROM readings WHERE category=%d AND url=%s""", category, url)
        if cursor.rowcount == 0:
            cursor.execute("""INSERT INTO readings VALUES(NULL, %d, %s)""", category, url)
            cursor.execute("""SELECT id FROM readings WHERE category=%d AND url=%s""", category, url)
        id = int(cursor.fetchone()[0])
        cursor.execute("""SELECT id FROM readers WHERE id=%d AND normalized=%s""", id, normalized)
        if cursor.rowcount == 0:
            cursor.execute("""INSERT INTO readers VALUES(%d, %s, %s)""", id, name, normalized)
        db.commit()

    def remove_reading(self, channel, category, name, url):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT id FROM readings WHERE category=%d AND url=%s""", category, url)
        if cursor.rowcount != 0:
            id = int(cursor.fetchone()[0])
            cursor.execute("""DELETE FROM readers WHERE id=%d AND normalized=%s""", id, normalized)
            cursor.execute("""SELECT name FROM readers WHERE id=%d""", id)
            if cursor.rowcount == 0:
                cursor.execute("""DELETE FROM readings WHERE id=%d""", id)
        db.commit()

    def get_random_reading(self, channel, category):
        db = self._getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT id FROM readings WHERE category=%d ORDER BY random() LIMIT 1""", category)
        if cursor.rowcount == 0:
            return None
        else:
            return int(cursor.fetchone()[0])

    def get_random_reading_containing(self, channel, category, substr):
        db = self._getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT id FROM readings WHERE category=%d AND url LIKE %s ORDER BY random() LIMIT 1""", category, '%'+substr+'%')
        if cursor.rowcount == 0:
            return None
        else:
            return int(cursor.fetchone()[0])

    def get_reading_url(self, channel, id):
        db = self._getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT url FROM readings WHERE id=%d""", id)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def get_reading_readers(self, channel, id):
        db = self._getDb(channel)
        cursor = db.cursor()
        cursor.execute("""SELECT name FROM readers WHERE id=%d""", id)
        if cursor.rowcount == 0:
            return None
        else:
            return [name for (name,) in cursor.fetchall()]

ReadingListDB = plugins.DB('ReadingList', {'sqlite': SqliteReadingListDB})

class ReadingList(callbacks.Plugin):
    """This plugin contains commands for maintaining a list of articles
    and other materials users have found interesting."""
    pass

    def __init__(self, irc):
        self.__parent = super(ReadingList, self)
        self.__parent.__init__(irc)
        self.db = ReadingListDB()

    def die(self):
        self.__parent.die()
        self.db.close()

    def reading(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Adds a URL to the list of interesting articles.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.add_reading(channel, 1, irc.msg.nick, url)
        irc.reply('added!')
    reading = wrap(reading, ['channel', 'text'])

    def unreading(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Removes a URL from the list of interesting articles.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.remove_reading(channel, 1, irc.msg.nick, url)
        irc.reply('removed!')
    unreading = wrap(unreading, ['channel', 'text'])

    def read(self, irc, msg, args, channel, substr):
        """[<channel>] [<substr>]
        
        Returns a URL from the list of interesting articles.
        If <substr> is given, only URLs containing <substr> are returned.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        id = 0
        if substr is None or substr == '':
            id = self.db.get_random_reading(channel, 1)
        else:
            id = self.db.get_random_reading_containing(channel, 1, substr)
        if id is None:
            irc.reply('Sorry, I don\'t have anything interesting for you to read.')
        else:
            url = self.db.get_reading_url(channel, id)
            readers = self.db.get_reading_readers(channel, id)
            if len(readers) == 1:
                irc.reply(format('%s suggests %s', str(readers[0]), url))
            else:
                irc.reply(format('%L suggest %s', readers, url))
    read = wrap(read, ['channel', additional('text')])

    def watching(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Adds a URL to the list of interesting videos.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.add_reading(channel, 2, irc.msg.nick, url)
        irc.reply('added!')
    watching = wrap(watching, ['channel', 'text'])

    def unwatching(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Removes a URL from the list of interesting videos.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.remove_reading(channel, 2, irc.msg.nick, url)
        irc.reply('removed!')
    unwatching = wrap(unwatching, ['channel', 'text'])

    def watch(self, irc, msg, args, channel, substr):
        """[<channel>] [<substr>]
        
        Returns a URL from the list of interesting videos.
        If <substr> is given, only URLs containing <substr> are returned.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        id = 0
        if substr is None or substr == '':
            id = self.db.get_random_reading(channel, 2)
        else:
            id = self.db.get_random_reading_containing(channel, 2, substr)
        if id is None:
            irc.reply('Sorry, I don\'t have anything interesting for you to watch.')
        else:
            url = self.db.get_reading_url(channel, id)
            readers = self.db.get_reading_readers(channel, id)
            if len(readers) == 1:
                irc.reply(format('%s suggests %s', str(readers[0]), url))
            else:
                irc.reply(format('%L suggest %s', readers, url))
    watch = wrap(watch, ['channel', additional('text')])

    def playing(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Adds a URL to the list of interesting games.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.add_reading(channel, 3, irc.msg.nick, url)
        irc.reply('added!')
    playing = wrap(playing, ['channel', 'text'])

    def unplaying(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Removes a URL from the list of interesting games.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.remove_reading(channel, 3, irc.msg.nick, url)
        irc.reply('removed!')
    unplaying = wrap(unplaying, ['channel', 'text'])

    def play(self, irc, msg, args, channel, substr):
        """[<channel>] [<substr>]
        
        Returns a URL from the list of interesting games.
        If <substr> is given, only URLs containing <substr> are returned.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        id = 0
        if substr is None or substr == '':
            id = self.db.get_random_reading(channel, 3)
        else:
            id = self.db.get_random_reading_containing(channel, 3, substr)
        if id is None:
            irc.reply('Sorry, I don\'t have anything interesting for you to play.')
        else:
            url = self.db.get_reading_url(channel, id)
            readers = self.db.get_reading_readers(channel, id)
            if len(readers) == 1:
                irc.reply(format('%s suggests %s', str(readers[0]), url))
            else:
                irc.reply(format('%L suggest %s', readers, url))
    play = wrap(play, ['channel', additional('text')])

    def learning(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Adds a URL to the list of interesting learning materials.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.add_reading(channel, 4, irc.msg.nick, url)
        irc.reply('added!')
    learning = wrap(learning, ['channel', 'text'])

    def unlearning(self, irc, msg, args, channel, url):
        """[<channel>] <url>

        Removes a URL from the list of interesting learning materials.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        self.db.remove_reading(channel, 4, irc.msg.nick, url)
        irc.reply('removed!')
    unlearning = wrap(unlearning, ['channel', 'text'])

    def learn(self, irc, msg, args, channel, substr):
        """[<channel>] [<substr>]
        
        Returns a URL from the list of interesting learning materials.
        If <substr> is given, only URLs containing <substr> are returned.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        id = 0
        if substr is None or substr == '':
            id = self.db.get_random_reading(channel, 4)
        else:
            id = self.db.get_random_reading_containing(channel, 4, substr)
        if id is None:
            irc.reply('Sorry, I don\'t have anything interesting for you to learn.')
        else:
            url = self.db.get_reading_url(channel, id)
            readers = self.db.get_reading_readers(channel, id)
            if len(readers) == 1:
                irc.reply(format('%s suggests %s', str(readers[0]), url))
            else:
                irc.reply(format('%L suggest %s', readers, url))
    learn = wrap(learn, ['channel', additional('text')])

Class = ReadingList

# vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
